<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: GetRequiredAdImpressionsType1.php 4154 2006-02-01 11:38:40Z andrew@m3.net $
*/

require_once MAX_PATH . '/lib/core/ServiceLocator.php';
require_once MAX_PATH . '/lib/max/Maintenance/Priority/AdServer/Task.php';
require_once MAX_PATH . '/lib/max/Maintenance/Priority/Criteria.php';
require_once MAX_PATH . '/lib/max/Maintenance/Priority/DeliveryLimitation.php';
require_once MAX_PATH . '/lib/max/Maintenance/Priority/Entities.php';
require_once 'Date.php';

/**
 * A class used to calculate the number of required advertisements in the next
 * operation interval for each ad, such that the ad will meet its delivery
 * requirements.
 *
 * This class is for advertisements in campaigns with Inventory Requirements and
 * a Campaign Expiration Date. The required impressions are calculated using the
 * following algorithm:
 *
 * - If the advertisement is in a campaign that has Click and/or Connection/Conversion
 *   Inventory Requirements, then calculate the campaign's Click Through Ratio and/or
 *   Connection/Conversion Ratio as required, and convert the campaign's remaining
 *   Click and/or Connection/Conversion Inventory Requirements into an equivalent
 *   Impression Inventory Requirement.
 * - Using the smallest of the real or calculated Impression Inventory Requirement values,
 *   distribute the required impressions between the campaign's advertisements, on the
 *   basis of the campaign's advertisement's weights.
 * - For each advertisement, divide the number of impressions allocated equally between
 *   the available operation intervals remaining in the day(s) before the end of the
 *   campaign. Do not include any operation intervals where the advertisement has been
 *   blocked from delivering (ie. a Day, Time, or Date Delivery Limitation exists). The
 *   number of impressions for each remaining operation interval will be the number of
 *   impressions required for the next operation interval, unless the advertisement is
 *   blocked from delivering in the next operation interval, in which case the required
 *   impressions for the next operation interval will be zero.
 *
 * @package    MaxMaintenance
 * @subpackage Priority
 * @author     Andrew Hill <andrew@m3.net>
 * @author     Demian Turner <demian@m3.net>
 * @author     James Floyd <james@m3.net>
 */
class GetRequiredAdImpressionsType1 extends MAX_Maintenance_Priority_AdServer_Task
{

    var $table;

    function getRequiredAdImpressionsType1()
    {
        parent::MAX_Maintenance_Priority_Adserver_Task();
        $this->table = &$this->_getMaxTablePriorityObj();
    }

    /**
     * The main method of the class, that is run by the controlling
     * task runner class.
     */
    function run()
    {
        MAX::debug('Starting to Get Required Ad Impressions.', PEAR_LOG_DEBUG);
        $aAllPlacements = $this->getValidPlacements();
        if (is_array($aAllPlacements) && (count($aAllPlacements) > 0)) {
            foreach ($aAllPlacements as $k => $oPlacement) {
                $this->getPlacementImpressionInventoryRequirement($aAllPlacements[$k]);
                $aAllPlacements[$k]->setAdverts();
            }
            $this->distributePlacementImpressions($aAllPlacements);
            //$this->distributePlacementImpressionsByZonePattern($aAllPlacements);
        }
    }

    /**
     * A method to return the current date. Can return the current date:
     *
     * - Based on the optional parameter; or
     * - From the current date registered with the Service Locator; or
     * - Based on the time when called.
     *
     * @access private
     * @param string $date An optional string representation of the current date.
     * @return object The current date.
     */
    function _getDate($date = '')
    {
        if (!empty($date)) {
            return new Date($date);
        }
        $oServiceLocator = &ServiceLocator::instance();
        $oDateNow = &$oServiceLocator->get('now');
        if (!$oDateNow) {
            $oDateNow = &new Date();
            $oServiceLocator->register('now', $oDateNow);
        }
        return $oDateNow;
    }

    /**
     * Method to return an array of Placement objects, given a set of filters.
     *
     * Essentially a convenience method to convert the results of the
     * {@link MAX_Dal_Maintenance::getPlacements()} method from an array of
     * database records into an array of Placement objects.
     *
     * @param array $fields An optional array of extra fields to select from the database
     *                      (see the {@link MAX_Dal_Maintenance::getPlacements()} class.)
     * @param array $wheres An optional array of where statements to limit which placements
     *                      are returned from the database (see the
     *                      {@link MAX_Dal_Maintenance::getPlacements()} class.)
     * @return array An array of Placement objects, appropriate to the $filter given.
     */
    function getAllPlacements($fields = array(), $wheres = array())
    {
        $aPlacementObjects = array();
        $aPlacements = $this->oDal->getPlacements($fields, $wheres);
        if (is_array($aPlacements) && (count($aPlacements) > 0)) {
            foreach ($aPlacements as $placement) {
                $aPlacementObjects[] = new Placement($placement);
            }
        }
        return $aPlacementObjects;
    }

    /**
     * A method that uses the getAllPlacements() method to obtain all placements
     * that meet the requirements of this class. That is:
     *
     * - The placement has an expiration date (that is not already passed); and
     * - As a result of the above, the placement must have an activation date that has
     *   passed, or has the default fake activation date; and
     * - The placement is active, and has a priority of at least 1; and
     * - The placement has inventory requirements for the duration of its activation.
     *
     * @return array An array of Placement objects.
     */
    function getValidPlacements()
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        // Get current date
        $oDate = $this->_getDate();
        $dateYMD = $oDate->format('%Y-%m-%d');
        $table = $conf['table']['prefix'] . $conf['table']['campaigns'];
        $wheres = array(
                        array("($table.activate = '0000-00-00' OR $table.activate <= '$dateYMD')", 'AND'),
                        array("$table.expire >= '$dateYMD'", 'AND'),
                        array("$table.priority >= 1", 'AND'),
                        array("$table.active = 't'", 'AND'),
                        array("($table.views > 0 OR $table.clicks > 0 OR $table.conversions > 0)", 'AND')
                       );
        return $this->getAllPlacements(array(), $wheres);
    }

    /**
     * Method to calculate the number of impressions required to achieve delivery
     * of a given click or conversion inventory.
     *
     * If historical delivery data does not exist, the ratio provided in $defaultRatio
     * will be use for all calculations.  Please be aware that a default a click ratio
     * and a default conversion rato are provided in the system wide configuration file
     * and should be a value between zero and one ((ratio > 0) && (ratio <= 1)).
     *
     * $GLOBALS['_MAX']['CONF']['priority']['defaultClickRatio']
     * $GLOBALS['_MAX']['CONF']['priority']['defaultConversionRatio']
     *
     * @param integer $inventory Total placement inventory to be achieved over lifetime/day.
     * @param integer $defaultRatio Click/conversion ratio used if historical data does not exist.
     * @param integer $inventoryToDate Total clicks/conversions achieved to date over lifetime/day.
     * @param integer $impressionsToDate Total impressions delivered to date over lifetime/day.
     * @return integer
     */
    function getInventoryImpressionsRequired($inventory, $defaultRatio, $inventoryToDate = 0, $impressionsToDate = 0)
    {
        $requiredImpressions = 0;
        if ($inventory > 0) {
            // If historical information exists
            if ($inventoryToDate > 0) {
                $inventoryPerImpression = ($inventoryToDate / $impressionsToDate);
            } else {
                $inventoryPerImpression = $defaultRatio;
            }
            $requiredImpressions = (int)$inventory / $inventoryPerImpression;
        }
        return ceil($requiredImpressions);
    }

    /**
     * Returns the smallest non zero value of a given array of values.
     *
     * If $aValues is empty (or contains no integer values), then zero is returned.
     *
     * @param array $aValues An array of values to evaluated.
     * @return integer The smallest non-zero value from the array, or zero if empty.
     */
    function getSmallestNonZeroInteger($aValues = array())
    {
        if (!is_array($aValues) || (count($aValues) == 0)) return 0;
        foreach ($aValues as $val) {
            if ((($val > 0) && !isset($minVal)) ||
                (($val > 0) && isset($minVal) && ($val < $minVal))) {
                $minVal = $val;
            }
        }
        return (isset($minVal)) ? $minVal : 0;
    }

    /**
     * Method to estimate the impressions required to fulfill a given
     * placement lifetime impression, click, or conversion requirement. If
     * more than one requirement exists the smallest calculated impression
     * requirement will be returned.
     *
     * The $oPlacement parameter is passed by reference and will have
     * the calculated impression requirement added to it in the position
     * $oPlacement->requiredImpressions
     *
     * @param object $oPlacement
     * @return void
     */
    function getPlacementImpressionInventoryRequirement(&$oPlacement)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        // Get campaign summary statistic totals
        $oPlacement->setSummaryStatisticsToDate();
        // Calculate impressions required to fulfill click requirement
        $clickImpressions = $this->getInventoryImpressionsRequired(
            $oPlacement->targetClicks,
            $conf['priority']['defaultClickRatio'],
            $oPlacement->sumClicks,
            $oPlacement->sumImpressions);
        // Calculate impressions required to fulfill conversion requirement
        $conversionImpressions = $this->getInventoryImpressionsRequired(
            $oPlacement->targetConversions,
            $conf['priority']['defaultConversionRatio'],
            $oPlacement->sumConversions,
            $oPlacement->sumImpressions);
        // Get impression requirement
        if ($oPlacement->targetImpressions > 0) {
            $impressions = $oPlacement->targetImpressions - $oPlacement->sumImpressions;
        } else {
            $impressions = 0;
        }
        // Choose smallest required impression
        $oPlacement->requiredImpressions = $this->getSmallestNonZeroInteger(
            array(
                $clickImpressions,
                $conversionImpressions,
                $impressions
            )
        );
    }

    /**
     * Method to sum all ad weight values of active advertisements for a given
     * placement. If no ads, or if the sum is zero, the default summation weight
     * is set to unity (1).
     *
     * @param object $oPlacement A Placement object.
     * @return integer The ad weight total.
     */
    function getPlacementAdWeightTotal($oPlacement)
    {
        $weight = 0;
        if (is_a($oPlacement, 'Placement')) {
            foreach ($oPlacement->adverts as $oAdvert) {
                if (is_a($oAdvert, 'Advert') && $oAdvert->active) {
                    $weight += ($oAdvert->weight > 0) ? $oAdvert->weight : 0;
                }
            }
            if ($weight === 0) {
                $weight = 1;
            }
            return $weight;
        }
        return PEAR::raiseError('Invalid Placement object argument', MAX_ERROR_INVALIDARGS);
    }

    /**
     * A method to distribute the calculated required placement impressions between the placement's
     * children advertisements. Impression allocation takes in to account ad weight, and the number
     * of operations intervals the ad will active in given date/time delivery constraints.
     *
     * The calculated ad impressions are written to the temporary table tmp_ad_required_impression
     * for later analysis by the {@link AllocateZoneImpressions} class.
     *
     * @param array $aPlacements An array of {@link Placement} objects which required that the
     *                           total required impressions be distributed between the component
     *                           advertisements.
     *
     * @TODO Update code to calculate required impressions for each ad for one week in advance,
     *       not just the next operation interval.
     * @TODO Check that campaign has delivery requirments $aCampaign['inventoryImpressionReq'] (?)
     */
    function distributePlacementImpressions($aPlacements)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        // Create an array for storing required ad impressions
        $aRequiredAdImpressions = array();
        // Get the current operation interval start/end
        $aCurrentOperationIntervalDates =
            $this->oOperationInt->convertDateToOperationIntervalStartAndEndDates($this->_getDate());
        // For each placement
        foreach ($aPlacements as $k => $oPlacement) {
            // Get date object to represent placement expire date
            if ($oPlacement->expire == '0000-00-00' &&
                (($oPlacement->dailyTargetImpressions > 0) ||
                 ($oPlacement->dailyTargetClicks > 0) ||
                 ($oPlacement->dailyTargetConversions > 0))) {
                // The campaign has a daily target, so treat this placement
                // as if it ends at the end of today
                $oDate = &$this->_getDate();
                // Get the end of the day from this date
                $oPlacementExpiryDate = &new Date($oDate->format('%Y-%m-%d') . ' 23:59:59');
            } else {
                $oPlacementExpiryDate = &$this->_getDate($oPlacement->expire);
                // Placement expires at end of expiry date so add one day less one
                // second so we have a date with time portion 23:59:59
                $oPlacementExpiryDate->addSeconds(SECONDS_PER_DAY - 1);
            }
            // Determine number of remaining operation intervals for placement
            $placementRemainingOperationIntervals =
                $this->oOperationInt->getRemaining(
                    $aCurrentOperationIntervalDates['start'],
                    $oPlacementExpiryDate
                );
            // Sum the weights of all (active) ads in placement
            $totalWeight = $this->getPlacementAdWeightTotal($oPlacement);
            if (PEAR::isError($totalWeight, MAX_ERROR_INVALIDARGS)) {
                /**
                 * @TODO Ensure that this PEAR::Error is handled by calling code, or
                 *       raise an error instead of returning.
                 */
                return $totalWeight;
            }
            // Calculate number impressions per weight value of 1 (one)
            $totalImpressionsPerUnitWeight = $oPlacement->requiredImpressions / $totalWeight;
            // For each (active) advertisement
            foreach ($oPlacement->adverts as $kk => $oAdvert) {
                // Get impressions required, based on the ad weight
                $requiredAdImpressions = 0;
                if ($oAdvert->active && ($oAdvert->weight > 0)) {
                    $requiredAdImpressions = $oAdvert->weight * $totalImpressionsPerUnitWeight;
                }
                if ($requiredAdImpressions > 0) {
                    $aDeliveryLimitations = $oAdvert->getDeliveryLimitations();
                    $oCriteria = new MAX_Maintenance_Priority_Criteria($aDeliveryLimitations);
                    if ($oCriteria->isLimitedNow($aCurrentOperationIntervalDates) === false) {
                        // Find number of active operation intervals
                        $activeAdOpInts =
                            $oCriteria->getActiveAdOperationIntervals(
                                $placementRemainingOperationIntervals,
                                $aCurrentOperationIntervalDates,
                                $oPlacementExpiryDate
                            );
                        // Are there active intervals for the ad?
                        if ($activeAdOpInts > 0) {
                            // Divide number required impressions between active operation intervals
                            /**
                             * @TODO Decide which of round(), ceil() or floor() is correct logic.
                             */
                            $oAdvert->requiredImpressions =
                                round($requiredAdImpressions / $activeAdOpInts);

                            // Delivery hack
                            if ($conf['maintenance']['deliveryHack']) {
                                $oServiceLocator = &ServiceLocator::instance();
                                $oDateNow = &$oServiceLocator->get('now');
                                $hour = $oDateNow->getHour();
                                $factor = 0.22;
                                switch ($hour) {
                                    case 8:
                                        $factor = 0.9;
                                        break;
                                    case 9:
                                        $factor = 1.43;
                                        break;
                                    case 10:
                                        $factor = 1.78;
                                        break;
                                    case 11:
                                        $factor = 1.86;
                                        break;
                                    case 12:
                                        $factor = 2.35;
                                        break;
                                    case 13:
                                        $factor = 2.2;
                                        break;
                                    case 14:
                                        $factor = 1.66;
                                        break;
                                    case 15:
                                        $factor = 1.84;
                                        break;
                                    case 16:
                                        $factor = 1.86;
                                        break;
                                    case 17:
                                        $factor = 1.87;
                                        break;
                                    case 18:
                                        $factor = 2.08;
                                        break;
                                    case 19:
                                        $factor = 2.05;
                                        break;
                                    case 20:
                                        $factor = 1.52;
                                        break;
                                    case 21:
                                        $factor = 1.43;
                                        break;
                                    case 22:
                                        $factor = 1.42;
                                        break;
                                    case 23:
                                        $factor = 0.86;
                                        break;
                                }
                                $oAdvert->requiredImpressions =
                                    round(($requiredAdImpressions / $activeAdOpInts) * $factor);
                            }

                            $aRequiredAdImpressions[] = array(
                                'ad_id'                => $oAdvert->id,
                                'required_impressions' => $oAdvert->requiredImpressions
                            );
                        }
                    }
                }
            }
        }
        // Save the required impressions into the temporary database table
        $this->table->createTable('tmp_ad_required_impression');
        $this->oDal->saveRequiredAdImpressions($aRequiredAdImpressions);
    }

    /**
     * A method to distribute the calculated required placement impressions between the placement's
     * children advertisements. Impression allocation takes in to account ad weight, and the number
     * of operations intervals the ad will be active in given date/time delivery limitations, and
     * the pattern of available impressions during the week for the zone(s) the advertisements are
     * linked to.
     *
     * The calculated ad impressions are written to the temporary table tmp_ad_required_impression
     * for later analysis by the {@link AllocateZoneImpressions} class.
     *
     * @param array $aPlacements An array of {@link Placement} objects which required that the
     *                           total required impressions be distributed between the component
     *                           advertisements.
     *
     * @param array $aPlacements
     */
    function distributePlacementImpressionsByZonePattern($aPlacements)
    {
        // Create an array for storing required ad impressions
        $aRequiredAdImpressions = array();
        // Get the current operation interval start/end
        $aCurrentOperationIntervalDates =
            $this->oOperationInt->convertDateToOperationIntervalStartAndEndDates($this->_getDate());
        // For each placement
        foreach ($aPlacements as $k => $oPlacement) {
            // Get date object to represent placement expire date
            if ($oPlacement->expire == '0000-00-00' &&
                (($oPlacement->target_impression > 0) ||
                 ($oPlacement->target_click > 0) ||
                 ($oPlacement->target_conversion > 0))) {
                // The campaign has a daily target, and no expiry date
                // treat this placement as if it ends at the end of today
                $oDate = &$this->_getDate();
                // Get the end of the day from this date
                $oPlacementExpiryDate = &new Date($oDate->format('%Y-%m-%d') . ' 23:59:59');
            } else {
                $oPlacementExpiryDate = &$this->_getDate($oPlacement->expire);
                // Placement expires at end of expiry date so add one day less one
                // second so we have a date with time portion 23:59:59
                $oPlacementExpiryDate->addSeconds(SECONDS_PER_DAY - 1);
            }
            // Sum the weights of all ads in placement
            $totalWeight = $this->getPlacementAdWeightTotal($oPlacement);
            if (PEAR::isError($totalWeight, MAX_ERROR_INVALIDARGS)) {
                /**
                 * @TODO Ensure that this PEAR::Error is handled by calling code, or
                 *       raise an error instead of returning.
                 */
                return $totalWeight;
            }
            // Calculate number impressions per weight value of 1 (one)
            $totalImpressionsPerUnitWeight = $oPlacement->requiredImpressions / $totalWeight;
            // For each advertisement
            foreach ($oPlacement->adverts as $kk => $oAdvert) {
                // If the advertisement is active, and has a positive weight
                if ($oAdvert->active && ($oAdvert->weight > 0)) {
                    // Calculate the impressions the ad requires, based on the ad weight
                    $totalRequiredAdImpressions = $oAdvert->weight * $totalImpressionsPerUnitWeight;
                    if ($totalRequiredAdImpressions > 0) {
                        // Based on the average zone pattern of the zones the ad is
                        // linked to, calculate how many of these impressions should
                        // be delivered in the next operation interval
                        $requiredAdImpressions =
                            $this->_getAdImpressions(
                                $oAdvert,
                                $totalRequiredAdImpressions,
                                $aCurrentOperationIntervalDates,
                                $oPlacementExpiryDate
                            );
                        // Set required impressions property of ad
                        $oAdvert->requiredImpressions = $requiredAdImpressions;
                        $aRequiredAdImpressions[] = array(
                            'ad_id'                => $oAdvert->id,
                            'required_impressions' => $oAdvert->requiredImpressions
                        );
                    }
                }
            }
        }
        // Save the required impressions into the temporary database table
        $this->table->createTable('tmp_ad_required_impression');
        $this->oDal->saveRequiredAdImpressions($aRequiredAdImpressions);
    }



    /**
     * A method to calcuate the number of impressions an ad needs to deliver in the next
     * operation interval, based on the total number of impressions the ad needs to deliver
     * over the rest of the campaign, the operaion intervals the ad will be active in,
     * and the average zone pattern of the zones the ad is linked to.
     *
     * @param Advert $oAdvert A reference to the Advert object.
     * @param integer $totalRequiredAdImpressions The total number of impressions the advertisement
     *                                            needs to deliver.
     * @param array $aDates An array containing Date objects representing the 'start' and 'end' of the
     *                      current operation interval.
     * @param Date $oPlacementExpiryDate A Date object representing the end date of the advertisement's
     *                                   parent placement.
     * @return integer The number of impressions the advertisement should deliver in the next
     *                 operation interval.
     *
     * @todo remove static call to OperationInterval::singleton();, modify test to use $this->oOperationInt
     */
    function _getAdImpressions(&$oAdvert, $totalRequiredAdImpressions, $aDates, $oPlacementExpiryDate)
    {
        $aDeliveryLimitations = $oAdvert->getDeliveryLimitations();
        $oCriteria = new MAX_Maintenance_Priority_Criteria($aDeliveryLimitations);
        if ($oCriteria->isLimitedNow($aDates) === true) {
            // The advertisement is not currently able to deliver, and so
            // no impressions should be allocated for this operation interval
            return 0;
        }
        // Get ad associated zones cumulative forecasts for previous week
        // keyed by operation interval id
        $aCumulativeZoneForecast = $this->_getCumulativeZoneForecast($oAdvert->id);
        // Get array representation of delivery availability day by day
        $aPlacementLife =
            $oCriteria->getInactiveOperationIntervals(
                $aDates['start'],
                $oPlacementExpiryDate
            );
        $oOperationInt = OperationInterval::singleton();
        $operationIntervalIdNow = $oOperationInt->getIntervalID($aDates['start']);
        $activeForecastTotal = 0;
        // Map of placement life as limitations exist
        if (is_array($aPlacementLife) && count($aPlacementLife)) {
            // For each day remaining check each interval
            foreach($aPlacementLife as $day) {
                $oi = $oOperationInt->getDayIntervalNumber($day[0]);
                // Get operation interval Id for date
                $operationIntervalId = $oOperationInt->getIntervalID($day[0]);
                for ($oi; $oi < $oOperationInt->intervalsPerDay(); $oi++) {
                    // If bit representing hour $oi is not set
                    if (!($day[1] & pow(2, $oi))) {
                        // Ad will deliver this interval add forecast to total
                        $activeForecastTotal += $aAdZoneForecast[$operationIntervalId];
                    }
                    $operationIntervalId++;
                }
            }
            // The number is.....
            if ($activeForecastTotal) {
                return $aAdZoneForecast[$operationIntervalIdNow] / $activeForecastTotal;
            } else {
                Max::debug("\$sumActiveForecast is zero or negative.", PEAR_LOG_ERR);
            }
        }
        return false;
    }

    /**
     * A method to return the current cumulative zone forecast data for all zones associated
     * with a given advertisement. The returned array is keyed by operation_interval_id (i.e.
     * from 0 [zero] to the maximum operation interval ID value, depending on the current
     * configuration value for the operation interval length). The zone forecast values used
     * in calculating the cumulative forecast are taken from the end of the current operation
     * interval to one week prior (i.e. the most recent week's worth of forecasts).
     *
     * @param integer $adId The advertisement ID.
     * @return mixed Array on success, false on failure. If an array, it is of the format:
     *                  array(
     *                      [operation_interval_id] => forecast_impressions,
     *                      [operation_interval_id] => forecast_impressions
     *                                  .
     *                                  .
     *                                  .
     *                  )
     */
    function _getCumulativeZoneForecast($adId)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        if (empty($adId) || !is_numeric($adId)) {
            MAX::debug("Invalid advertisement ID argument", PEAR_LOG_ERR);
            return false;
        }
        // Get all zones associated with the advertisement
        $aAdZones = $this->oDal->getAdZoneAssociationsByAds(array($adId));
        $aZones = @$aAdZones[$adId];
        // Initialise the results array with the number operation intervals in a week
        $aResults = array_fill(0, $this->oOperationInt->intervalsPerWeek(), 0);
        // Get the forcast impressions for the previous week
        if (is_array($aZones) && !empty($aZones)) {
            foreach ($aZones as $aZone) {
                $aForecast = $this->oDal->getPreviousWeekZoneForcastImpressions($aZone['zone_id']);
                if (is_array($aForecast) && !empty($aForecast)) {
                    foreach ($aForecast as $aValues) {
                        $aResults[$aValues['operation_interval_id']] += (int)$aValues['forecast_impressions'];;
                    }
                }
            }
        }
        return $aResults;
    }

}

?>
